"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var p = _interopRequireWildcard(require("path"));

var _fs = require("fs");

var _mkdirp = require("mkdirp");

var _printIcuMessage = _interopRequireDefault(require("babel-plugin-react-intl/lib/print-icu-message"));

var _flowRuntime = _interopRequireDefault(require("flow-runtime"));

/* eslint-disable no-param-reassign */

/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */
const COMPONENT_NAMES = ['FormattedMessage', 'FormattedHTMLMessage'];
const FUNCTION_NAMES = ['defineMessages'];
const DESCRIPTOR_PROPS = new Set(['id', 'description', 'defaultMessage']);
const EXTRACTED = Symbol('ReactIntlExtracted');
const MESSAGES = Symbol('ReactIntlMessages');

var _default = _flowRuntime.default.annotate(function (_arg) {
  let {
    types: t
  } = _flowRuntime.default.ref("$FixMe").assert(_arg);

  function getModuleSourceName(opts) {
    return opts.moduleSourceName || 'react-intl';
  }

  function evaluatePath(path) {
    const evaluated = path.evaluate();

    if (evaluated.confident) {
      return evaluated.value;
    }

    throw path.buildCodeFrameError('[React Intl] Messages must be statically evaluate-able for extraction.');
  }

  function getMessageDescriptorKey(path) {
    if (path.isIdentifier() || path.isJSXIdentifier()) {
      return path.node.name;
    }

    return evaluatePath(path);
  }

  function getMessageDescriptorValue(path) {
    if (path.isJSXExpressionContainer()) {
      path = path.get('expression');
    } // Always trim the Message Descriptor values.


    const descriptorValue = evaluatePath(path);

    if (typeof descriptorValue === 'string') {
      return descriptorValue.trim();
    }

    return descriptorValue;
  }

  function getICUMessageValue(messagePath, {
    isJSXSource = false
  } = {}) {
    const message = getMessageDescriptorValue(messagePath);

    try {
      return (0, _printIcuMessage.default)(message);
    } catch (parseError) {
      if (isJSXSource && messagePath.isLiteral() && message.indexOf('\\\\') >= 0) {
        throw messagePath.buildCodeFrameError('[React Intl] Message failed to parse. ' + 'It looks like `\\`s were used for escaping, ' + "this won't work with JSX string literals. " + 'Wrap with `{}`. ' + 'See: http://facebook.github.io/react/docs/jsx-gotchas.html');
      }

      throw messagePath.buildCodeFrameError('[React Intl] Message failed to parse. ' + 'See: http://formatjs.io/guides/message-syntax/' + `\n${parseError}`);
    }
  }

  function createMessageDescriptor(propPaths) {
    return propPaths.reduce((hash, [keyPath, valuePath]) => {
      const key = getMessageDescriptorKey(keyPath);

      if (DESCRIPTOR_PROPS.has(key)) {
        hash[key] = valuePath;
      }

      return hash;
    }, {});
  }

  function evaluateMessageDescriptor(_ref, {
    isJSXSource = false
  } = {}) {
    let descriptor = (0, _extends2.default)({}, _ref);
    Object.keys(descriptor).forEach(key => {
      const valuePath = descriptor[key];

      if (key === 'defaultMessage') {
        descriptor[key] = getICUMessageValue(valuePath, {
          isJSXSource
        });
      } else {
        descriptor[key] = getMessageDescriptorValue(valuePath);
      }
    });
    return descriptor;
  }

  function storeMessage({
    id,
    description,
    defaultMessage
  }, path, state) {
    const {
      file,
      opts
    } = state;

    if (!(id && defaultMessage)) {
      throw path.buildCodeFrameError('[React Intl] Message Descriptors require an `id` and `defaultMessage`.');
    }

    const messages = file.get(MESSAGES);

    if (messages.has(id)) {
      const existing = messages.get(id);

      if (description !== existing.description || defaultMessage !== existing.defaultMessage) {
        throw path.buildCodeFrameError(`[React Intl] Duplicate message id: "${id}", ` + 'but the `description` and/or `defaultMessage` are different.');
      }
    }

    if (opts.enforceDescriptions) {
      if (!description || typeof description === 'object' && Object.keys(description).length < 1) {
        throw path.buildCodeFrameError('[React Intl] Message must have a `description`.');
      }
    }

    let loc = null;

    if (opts.extractSourceLocation) {
      loc = (0, _objectSpread2.default)({
        file: p.relative(process.cwd(), file.opts.filename)
      }, path.node.loc);
    }

    messages.set(id, (0, _objectSpread2.default)({
      id,
      description,
      defaultMessage
    }, loc));
  }

  function referencesImport(path, mod, importedNames) {
    if (!(path.isIdentifier() || path.isJSXIdentifier())) {
      return false;
    }

    return importedNames.some(name => path.referencesImport(mod, name));
  }

  function tagAsExtracted(path) {
    path.node[EXTRACTED] = true;
  }

  function wasExtracted(path) {
    return Boolean(path.node[EXTRACTED]);
  }

  return {
    pre(file) {
      let _fileType = _flowRuntime.default.ref("$FixMe");

      _flowRuntime.default.param("file", _fileType).assert(file);

      if (!file.has(MESSAGES)) {
        file.set(MESSAGES, new Map());
      }
    },

    post(file) {
      let _fileType2 = _flowRuntime.default.ref("$FixMe");

      _flowRuntime.default.param("file", _fileType2).assert(file);

      const {
        opts
      } = this;
      const {
        filename
      } = file.opts;
      const basename = p.basename(filename, p.extname(filename));
      const messages = file.get(MESSAGES);
      const descriptors = [...messages.values()];
      file.metadata['react-intl'] = {
        messages: descriptors
      };

      if (opts.messagesDir && descriptors.length > 0) {
        // Make sure the relative path is "absolute" before
        // joining it with the `messagesDir`.
        const relativePath = p.join(p.sep, p.relative(process.cwd(), filename));
        const messagesFilename = p.join(opts.messagesDir, p.dirname(relativePath), `${basename}.json`);
        const messagesFile = JSON.stringify(descriptors, null, 2);
        (0, _mkdirp.sync)(p.dirname(messagesFilename));
        (0, _fs.writeFileSync)(messagesFilename, messagesFile);
      }
    },

    visitor: {
      JSXOpeningElement(path, state) {
        let _pathType = _flowRuntime.default.ref("$FixMe");

        let _stateType = _flowRuntime.default.ref("$FixMe");

        _flowRuntime.default.param("path", _pathType).assert(path);

        _flowRuntime.default.param("state", _stateType).assert(state);

        if (wasExtracted(path)) {
          return;
        }

        const {
          file,
          opts
        } = state;
        const moduleSourceName = getModuleSourceName(opts);
        const name = path.get('name');

        if (name.referencesImport(moduleSourceName, 'FormattedPlural')) {
          file.log.warn(`[React Intl] Line ${path.node.loc.start.line}: ` + 'Default messages are not extracted from ' + '<FormattedPlural>, use <FormattedMessage> instead.');
          return;
        }

        if (referencesImport(name, moduleSourceName, COMPONENT_NAMES)) {
          const attributes = path.get('attributes').filter(attr => attr.isJSXAttribute());
          let descriptor = createMessageDescriptor(attributes.map(attr => [attr.get('name'), attr.get('value')])); // In order for a default message to be extracted when
          // declaring a JSX element, it must be done with standard
          // `key=value` attributes. But it's completely valid to
          // write `<FormattedMessage {...descriptor} />` or
          // `<FormattedMessage id={dynamicId} />`, because it will be
          // skipped here and extracted elsewhere. The descriptor will
          // be extracted only if a `defaultMessage` prop exists.

          if (descriptor.defaultMessage) {
            // Evaluate the Message Descriptor values in a JSX
            // context, then store it.
            descriptor = evaluateMessageDescriptor(descriptor, {
              isJSXSource: true
            });
            storeMessage(descriptor, path, state); // Remove description since it's not used at runtime.

            attributes.some(attr => {
              const ketPath = attr.get('name');

              if (getMessageDescriptorKey(ketPath) === 'description') {
                attr.remove();
                return true;
              }

              return false;
            }); // Tag the AST node so we don't try to extract it twice.

            tagAsExtracted(path);
          }
        }
      },

      CallExpression(path, state) {
        let _pathType2 = _flowRuntime.default.ref("$FixMe");

        let _stateType2 = _flowRuntime.default.ref("$FixMe");

        _flowRuntime.default.param("path", _pathType2).assert(path);

        _flowRuntime.default.param("state", _stateType2).assert(state);

        const moduleSourceName = getModuleSourceName(state.opts);
        const callee = path.get('callee');

        function assertObjectExpression(node) {
          if (!(node && node.isObjectExpression())) {
            throw path.buildCodeFrameError(`[React Intl] \`${callee.node.name}()\` must be ` + 'called with an object expression with values ' + 'that are React Intl Message Descriptors, also ' + 'defined as object expressions.');
          }
        }

        function processMessageObject(messageObj) {
          assertObjectExpression(messageObj);

          if (wasExtracted(messageObj)) {
            return;
          }

          const properties = messageObj.get('properties');
          let descriptor = createMessageDescriptor(properties.map(prop => [prop.get('key'), prop.get('value')])); // Evaluate the Message Descriptor values, then store it.

          descriptor = evaluateMessageDescriptor(descriptor);
          storeMessage(descriptor, messageObj, state); // Remove description since it's not used at runtime.

          messageObj.replaceWith(t.objectExpression([t.objectProperty(t.stringLiteral('id'), t.stringLiteral(descriptor.id)), t.objectProperty(t.stringLiteral('defaultMessage'), t.stringLiteral(descriptor.defaultMessage))])); // Tag the AST node so we don't try to extract it twice.

          tagAsExtracted(messageObj);
        }

        if (referencesImport(callee, moduleSourceName, FUNCTION_NAMES)) {
          const [messagesObj] = path.get('arguments');
          assertObjectExpression(messagesObj);
          messagesObj.get('properties').map(prop => prop.get('value')).forEach(processMessageObject);
        }

        if (isIntlFunction(callee)) {
          const [messagesObj] = path.get('arguments');

          try {
            assertObjectExpression(messagesObj);
            processMessageObject(messagesObj);
          } catch (e) {// console.log(e);
          }
        }
      }

    }
  };
}, _flowRuntime.default.function(_flowRuntime.default.param("_arg", _flowRuntime.default.ref("$FixMe"))));

exports.default = _default;

function isIntlFunction(path) {
  if (path.node.type !== 'MemberExpression') {
    return false;
  }

  return path.node.property.name === 'formatMessage' || path.node.property.name === 'formatHTMLMessage';
}